=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2011-02-08 15:35:14 +0000
+++ bzrlib/branch.py	2011-02-18 20:28:34 +0000
@@ -880,13 +880,14 @@
                 # XXX: If you unstack a branch while it has a working tree
                 # with a pending merge, the pending-merged revisions will no
                 # longer be present.  You can (probably) revert and remerge.
-                #
-                # XXX: This only fetches up to the tip of the repository; it
-                # doesn't bring across any tags.  That's fairly consistent
-                # with how branch works, but perhaps not ideal.
-                self.repository.fetch(old_repository,
-                    revision_id=self.last_revision(),
-                    find_ghosts=True)
+                try:
+                    tags_to_fetch = set(self.tags.get_reverse_tag_dict())
+                except errors.TagsNotSupported:
+                    tags_to_fetch = set()
+                fetch_spec = _mod_graph.NotInOtherForRevs(self.repository,
+                    old_repository, required_ids=[self.last_revision()],
+                    if_present_ids=tags_to_fetch, find_ghosts=True).execute()
+                self.repository.fetch(old_repository, fetch_spec=fetch_spec)
             finally:
                 old_repository.unlock()
         finally:

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-02-09 17:10:05 +0000
+++ bzrlib/builtins.py	2011-02-18 12:00:14 +0000
@@ -6103,7 +6103,7 @@
     """
     takes_options = ['directory',
                      Option('ignored', help='Delete all ignored files.'),
-                     Option('detritus', help='Delete conflict files, merge'
+                     Option('detritus', help='Delete conflict files, merge and revert'
                             ' backups, and failed selftest dirs.'),
                      Option('unknown',
                             help='Delete files unknown to bzr (default).'),

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2011-02-08 15:35:14 +0000
+++ bzrlib/bzrdir.py	2011-02-18 20:28:34 +0000
@@ -185,6 +185,7 @@
         :param create_prefix: Create any missing directories leading up to
             to_transport.
         :param use_existing_dir: Use an existing directory if one exists.
+        :param no_tree: If set to true prevents creation of a working tree.
         """
         # Overview: put together a broad description of what we want to end up
         # with; then make as few api calls as possible to do it.

=== renamed file 'bzrlib/thread.py' => 'bzrlib/cethread.py'
--- bzrlib/thread.py	2011-02-08 16:26:23 +0000
+++ bzrlib/cethread.py	2011-02-10 12:37:27 +0000
@@ -39,6 +39,7 @@
         self.set_sync_event(sync_event)
         self.exception = None
         self.ignored_exceptions = None # see set_ignored_exceptions
+        self.lock = threading.Lock()
 
     # compatibility thunk for python-2.4 and python-2.5...
     if sys.version_info < (2, 6):
@@ -55,11 +56,49 @@
         Some threads require multiple events and should set the relevant one
         when appropriate.
 
-        Note that the event should be cleared so the caller can wait() on him
-        and be released when the thread set the event.
+        Note that the event should be initially cleared so the caller can
+        wait() on him and be released when the thread set the event.
+
+        Also note that the thread can use multiple events, setting them as it
+        progress, while the caller can chose to wait on any of them. What
+        matters is that there is always one event set so that the caller is
+        always released when an exception is caught. Re-using the same event is
+        therefore risky as the thread itself has no idea about which event the
+        caller is waiting on. If the caller has already been released then a
+        cleared event won't guarantee that the caller is still waiting on it.
         """
         self.sync_event = event
 
+    def switch_and_set(self, new):
+        """Switch to a new ``sync_event`` and set the current one.
+
+        Using this method protects against race conditions while setting a new
+        ``sync_event``.
+
+        Note that this allows a caller to wait either on the old or the new
+        event depending on whether it wants a fine control on what is happening
+        inside a thread.
+
+        :param new: The event that will become ``sync_event``
+        """
+        cur = self.sync_event
+        self.lock.acquire()
+        try: # Always release the lock
+            try:
+                self.set_sync_event(new)
+                # From now on, any exception will be synced with the new event
+            except:
+                # Unlucky, we couldn't set the new sync event, try restoring a
+                # safe state
+                self.set_sync_event(cur)
+                raise
+            # Setting the current ``sync_event`` will release callers waiting
+            # on it, note that it will also be set in run() if an exception is
+            # raised
+            cur.set()
+        finally:
+            self.lock.release()
+
     def set_ignored_exceptions(self, ignored):
         """Declare which exceptions will be ignored.
 

=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2011-02-07 04:14:29 +0000
+++ bzrlib/controldir.py	2011-02-16 15:52:12 +0000
@@ -602,7 +602,7 @@
 
     def clone_on_transport(self, transport, revision_id=None,
         force_new_repo=False, preserve_stacking=False, stacked_on=None,
-        create_prefix=False, use_existing_dir=True):
+        create_prefix=False, use_existing_dir=True, no_tree=False):
         """Clone this bzrdir and its contents to transport verbatim.
 
         :param transport: The transport for the location to produce the clone
@@ -617,6 +617,7 @@
         :param create_prefix: Create any missing directories leading up to
             to_transport.
         :param use_existing_dir: Use an existing directory if one exists.
+        :param no_tree: If set to true prevents creation of a working tree.
         """
         raise NotImplementedError(self.clone_on_transport)
 

=== modified file 'bzrlib/decorators.py'
--- bzrlib/decorators.py	2010-02-17 17:11:16 +0000
+++ bzrlib/decorators.py	2011-02-14 11:57:45 +0000
@@ -30,14 +30,16 @@
 def _get_parameters(func):
     """Recreate the parameters for a function using introspection.
 
-    :return: (function_params, calling_params)
+    :return: (function_params, calling_params, default_values)
         function_params: is a string representing the parameters of the
             function. (such as "a, b, c=None, d=1")
             This is used in the function declaration.
         calling_params: is another string representing how you would call the
             function with the correct parameters. (such as "a, b, c=c, d=d")
-            Assuming you sued function_params in the function declaration, this
+            Assuming you used function_params in the function declaration, this
             is the parameters to put in the function call.
+        default_values_block: a dict with the default values to be passed as
+            the scope for the 'exec' statement.
 
         For example:
 
@@ -49,9 +51,15 @@
     # it globally.
     import inspect
     args, varargs, varkw, defaults = inspect.getargspec(func)
+    defaults_dict = {}
+    def formatvalue(value):
+        default_name = '__default_%d' % len(defaults_dict)
+        defaults_dict[default_name] = value
+        return '=' + default_name
     formatted = inspect.formatargspec(args, varargs=varargs,
                                       varkw=varkw,
-                                      defaults=defaults)
+                                      defaults=defaults,
+                                      formatvalue=formatvalue)
     if defaults is None:
         args_passed = args
     else:
@@ -65,7 +73,7 @@
         args_passed.append('**' + varkw)
     args_passed = ', '.join(args_passed)
 
-    return formatted[1:-1], args_passed
+    return formatted[1:-1], args_passed, defaults_dict
 
 
 def _pretty_needs_read_lock(unbound):
@@ -107,14 +115,17 @@
         return result
 read_locked = %(name)s_read_locked
 """
-    params, passed_params = _get_parameters(unbound)
+    params, passed_params, defaults_dict = _get_parameters(unbound)
     variables = {'name':unbound.__name__,
                  'params':params,
                  'passed_params':passed_params,
                 }
     func_def = template % variables
 
-    exec func_def in locals()
+    scope = dict(defaults_dict)
+    scope['unbound'] = unbound
+    exec func_def in scope
+    read_locked = scope['read_locked']
 
     read_locked.__doc__ = unbound.__doc__
     read_locked.__name__ = unbound.__name__
@@ -172,14 +183,17 @@
         return result
 write_locked = %(name)s_write_locked
 """
-    params, passed_params = _get_parameters(unbound)
+    params, passed_params, defaults_dict = _get_parameters(unbound)
     variables = {'name':unbound.__name__,
                  'params':params,
                  'passed_params':passed_params,
                 }
     func_def = template % variables
 
-    exec func_def in locals()
+    scope = dict(defaults_dict)
+    scope['unbound'] = unbound
+    exec func_def in scope
+    write_locked = scope['write_locked']
 
     write_locked.__doc__ = unbound.__doc__
     write_locked.__name__ = unbound.__name__

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2010-03-08 02:23:17 +0000
+++ bzrlib/export/zip_exporter.py	2011-02-16 17:20:10 +0000
@@ -37,9 +37,10 @@
 # Or it won't consider the entry a directory
 ZIP_DIRECTORY_BIT = (1 << 4)
 FILE_PERMISSIONS = (0644 << 16)
+DIR_PERMISSIONS = (0755 << 16)
 
 _FILE_ATTR = stat.S_IFREG | FILE_PERMISSIONS
-_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT
+_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT | DIR_PERMISSIONS
 
 
 def zip_exporter(tree, dest, root, subdir, filtered=False,

=== modified file 'bzrlib/groupcompress.py'
--- bzrlib/groupcompress.py	2011-02-09 06:35:00 +0000
+++ bzrlib/groupcompress.py	2011-02-14 02:37:04 +0000
@@ -1186,7 +1186,7 @@
             _unadded_refs = {}
         self._unadded_refs = _unadded_refs
         self._group_cache = LRUSizeCache(max_size=50*1024*1024)
-        self._fallback_vfs = []
+        self._immediate_fallback_vfs = []
 
     def without_fallbacks(self):
         """Return a clone of this object without any fallbacks configured."""
@@ -1266,7 +1266,7 @@
 
         :param a_versioned_files: A VersionedFiles object.
         """
-        self._fallback_vfs.append(a_versioned_files)
+        self._immediate_fallback_vfs.append(a_versioned_files)
 
     def annotate(self, key):
         """See VersionedFiles.annotate."""
@@ -1342,7 +1342,7 @@
             and so on.
         """
         result = {}
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         source_results = []
         missing = set(keys)
         for source in sources:
@@ -1449,7 +1449,7 @@
         parent_map = {}
         key_to_source_map = {}
         source_results = []
-        for source in self._fallback_vfs:
+        for source in self._immediate_fallback_vfs:
             if not missing:
                 break
             source_parents = source.get_parent_map(missing)
@@ -1832,7 +1832,7 @@
         """See VersionedFiles.keys."""
         if 'evil' in debug.debug_flags:
             trace.mutter_callsite(2, "keys scales with size of history")
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         result = set()
         for source in sources:
             result.update(source.keys())

=== modified file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	2010-09-28 07:34:29 +0000
+++ bzrlib/hooks.py	2011-02-18 17:44:53 +0000
@@ -115,9 +115,39 @@
     FOO hook is triggered.
     """
 
-    def __init__(self):
+    def __init__(self, module=None, member_name=None):
+        """Create a new hooks dictionary.
+
+        :param module: The module from which this hooks dictionary should be loaded
+            (used for lazy hooks)
+        :param member_name: Name under which this hooks dictionary should be loaded.
+            (used for lazy hooks)
+        """
         dict.__init__(self)
         self._callable_names = {}
+        self._module = module
+        self._member_name = member_name
+
+    def add_hook(self, name, doc, introduced, deprecated=None):
+        """Add a hook point to this dictionary.
+
+        :param name: The name of the hook, for clients to use when registering.
+        :param doc: The docs for the hook.
+        :param introduced: When the hook was introduced (e.g. (0, 15)).
+        :param deprecated: When the hook was deprecated, None for
+            not-deprecated.
+        """
+        if name in self:
+            raise errors.DuplicateKey(name)
+        if self._module:
+            callbacks = _lazy_hooks.setdefault(
+                (self._module, self._member_name, name), [])
+        else:
+            callbacks = None
+        hookpoint = HookPoint(name=name, doc=doc, introduced=introduced,
+                              deprecated=deprecated,
+                              callbacks=callbacks)
+        self[name] = hookpoint
 
     def create_hook(self, hook):
         """Create a hook which can have callbacks registered for it.
@@ -166,14 +196,38 @@
         """
         return self._callable_names.get(a_callable, "No hook name")
 
+    def install_named_hook_lazy(self, hook_name, callable_module,
+        callable_member, name):
+        """Install a_callable in to the hook hook_name lazily, and label it.
+
+        :param hook_name: A hook name. See the __init__ method for the complete
+            list of hooks.
+        :param callable_module: Name of the module in which the callable is
+            present.
+        :param callable_member: Member name of the callable.
+        :param name: A name to associate the callable with, to show users what
+            is running.
+        """
+        try:
+            hook = self[hook_name]
+        except KeyError:
+            raise errors.UnknownHook(self.__class__.__name__, hook_name)
+        try:
+            hook_lazy = getattr(hook, "hook_lazy")
+        except AttributeError:
+            raise errors.UnsupportedOperation(self.install_named_hook_lazy,
+                self)
+        else:
+            hook_lazy(callable_module, callable_member, name)
+
     def install_named_hook(self, hook_name, a_callable, name):
         """Install a_callable in to the hook hook_name, and label it name.
 
-        :param hook_name: A hook name. See the __init__ method of BranchHooks
-            for the complete list of hooks.
+        :param hook_name: A hook name. See the __init__ method for the complete
+            list of hooks.
         :param a_callable: The callable to be invoked when the hook triggers.
             The exact signature will depend on the hook - see the __init__
-            method of BranchHooks for details on each hook.
+            method for details on each hook.
         :param name: A name to associate a_callable with, to show users what is
             running.
         """
@@ -207,7 +261,7 @@
         should describe the recommended replacement hook to register for.
     """
 
-    def __init__(self, name, doc, introduced, deprecated):
+    def __init__(self, name, doc, introduced, deprecated=None, callbacks=None):
         """Create a HookPoint.
 
         :param name: The name of the hook, for clients to use when registering.
@@ -220,8 +274,10 @@
         self.__doc__ = doc
         self.introduced = introduced
         self.deprecated = deprecated
-        self._callbacks = []
-        self._callback_names = {}
+        if callbacks is None:
+            self._callbacks = []
+        else:
+            self._callbacks = callbacks
 
     def docs(self):
         """Generate the documentation for this HookPoint.
@@ -247,8 +303,20 @@
         return '\n'.join(strings)
 
     def __eq__(self, other):
-        return (type(other) == type(self) and 
-            other.__dict__ == self.__dict__)
+        return (type(other) == type(self) and other.__dict__ == self.__dict__)
+
+    def hook_lazy(self, callback_module, callback_member, callback_label):
+        """Lazily register a callback to be called when this HookPoint fires.
+
+        :param callback_module: Module of the callable to use when this
+            HookPoint fires.
+        :param callback_member: Member name of the callback.
+        :param callback_label: A label to show in the UI while this callback is
+            processing.
+        """
+        obj_getter = registry._LazyObjectGetter(callback_module,
+            callback_member)
+        self._callbacks.append((obj_getter, callback_label))
 
     def hook(self, callback, callback_label):
         """Register a callback to be called when this HookPoint fires.
@@ -257,12 +325,11 @@
         :param callback_label: A label to show in the UI while this callback is
             processing.
         """
-        self._callbacks.append(callback)
-        if callback_label is not None:
-            self._callback_names[callback] = callback_label
+        obj_getter = registry._ObjectGetter(callback)
+        self._callbacks.append((obj_getter, callback_label))
 
     def __iter__(self):
-        return iter(self._callbacks)
+        return (callback.get_obj() for callback, name in self._callbacks)
 
     def __len__(self):
         return len(self._callbacks)
@@ -272,12 +339,13 @@
         strings.append("<%s(" % type(self).__name__)
         strings.append(self.name)
         strings.append("), callbacks=[")
-        for callback in self._callbacks:
-            strings.append(repr(callback))
+        callbacks = self._callbacks
+        for (callback, callback_name) in callbacks:
+            strings.append(repr(callback.get_obj()))
             strings.append("(")
-            strings.append(self._callback_names[callback])
+            strings.append(callback_name)
             strings.append("),")
-        if len(self._callbacks) == 1:
+        if len(callbacks) == 1:
             strings[-1] = ")"
         strings.append("]>")
         return ''.join(strings)
@@ -322,3 +390,22 @@
         hooks = known_hooks_key_to_object(hook_key)
         segments.append(hooks.docs())
     return '\n'.join(segments)
+
+
+_lazy_hooks = {}
+
+
+def install_lazy_named_hook(hookpoints_module, hookpoints_name, hook_name,
+    a_callable, name):
+    """Install a callable in to a hook lazily, and label it name.
+
+    :param hookpoints_module: Module name of the hook points.
+    :param hookpoints_name: Name of the hook points.
+    :param hook_name: A hook name.
+    :param callable: a callable to call for the hook.
+    :param name: A name to associate a_callable with, to show users what is
+        running.
+    """
+    key = (hookpoints_module, hookpoints_name, hook_name)
+    obj_getter = registry._ObjectGetter(a_callable)
+    _lazy_hooks.setdefault(key, []).append((obj_getter, name))

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2011-02-09 06:35:00 +0000
+++ bzrlib/knit.py	2011-02-14 02:37:04 +0000
@@ -883,7 +883,7 @@
             self._factory = KnitAnnotateFactory()
         else:
             self._factory = KnitPlainFactory()
-        self._fallback_vfs = []
+        self._immediate_fallback_vfs = []
         self._reload_func = reload_func
 
     def __repr__(self):
@@ -897,7 +897,7 @@
 
         :param a_versioned_files: A VersionedFiles object.
         """
-        self._fallback_vfs.append(a_versioned_files)
+        self._immediate_fallback_vfs.append(a_versioned_files)
 
     def add_lines(self, key, parents, lines, parent_texts=None,
         left_matching_blocks=None, nostore_sha=None, random_id=False,
@@ -1070,7 +1070,7 @@
                     raise errors.KnitCorrupt(self,
                         "Missing basis parent %s for %s" % (
                         compression_parent, key))
-        for fallback_vfs in self._fallback_vfs:
+        for fallback_vfs in self._immediate_fallback_vfs:
             fallback_vfs.check()
 
     def _check_add(self, key, lines, random_id, check_content):
@@ -1226,7 +1226,7 @@
             and so on.
         """
         result = {}
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         source_results = []
         missing = set(keys)
         for source in sources:
@@ -1526,7 +1526,7 @@
                         yield KnitContentFactory(key, global_map[key],
                             record_details, None, raw_data, self._factory.annotated, None)
                 else:
-                    vf = self._fallback_vfs[parent_maps.index(source) - 1]
+                    vf = self._immediate_fallback_vfs[parent_maps.index(source) - 1]
                     for record in vf.get_record_stream(keys, ordering,
                         include_delta_closure):
                         yield record
@@ -1542,7 +1542,7 @@
             # record entry 2 is the 'digest'.
             result[key] = details[2]
         missing.difference_update(set(result))
-        for source in self._fallback_vfs:
+        for source in self._immediate_fallback_vfs:
             if not missing:
                 break
             new_result = source.get_sha1s(missing)
@@ -1619,7 +1619,7 @@
                 raise RevisionNotPresent([record.key], self)
             elif ((record.storage_kind in knit_types)
                   and (compression_parent is None
-                       or not self._fallback_vfs
+                       or not self._immediate_fallback_vfs
                        or self._index.has_key(compression_parent)
                        or not self.has_key(compression_parent))):
                 # we can insert the knit record literally if either it has no
@@ -1797,11 +1797,11 @@
         # vfs, and hope to find them there.  Note that if the keys are found
         # but had no changes or no content, the fallback may not return
         # anything.
-        if keys and not self._fallback_vfs:
+        if keys and not self._immediate_fallback_vfs:
             # XXX: strictly the second parameter is meant to be the file id
             # but it's not easily accessible here.
             raise RevisionNotPresent(keys, repr(self))
-        for source in self._fallback_vfs:
+        for source in self._immediate_fallback_vfs:
             if not keys:
                 break
             source_keys = set()
@@ -2016,7 +2016,7 @@
         """See VersionedFiles.keys."""
         if 'evil' in debug.debug_flags:
             trace.mutter_callsite(2, "keys scales with size of history")
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         result = set()
         for source in sources:
             result.update(source.keys())
@@ -2062,7 +2062,7 @@
 
         missing_keys = set(nonlocal_keys)
         # Read from remote versioned file instances and provide to our caller.
-        for source in self.vf._fallback_vfs:
+        for source in self.vf._immediate_fallback_vfs:
             if not missing_keys:
                 break
             # Loop over fallback repositories asking them for texts - ignore
@@ -3521,8 +3521,8 @@
         return records, ann_keys
 
     def _get_needed_texts(self, key, pb=None):
-        # if True or len(self._vf._fallback_vfs) > 0:
-        if len(self._vf._fallback_vfs) > 0:
+        # if True or len(self._vf._immediate_fallback_vfs) > 0:
+        if len(self._vf._immediate_fallback_vfs) > 0:
             # If we have fallbacks, go to the generic path
             for v in annotate.Annotator._get_needed_texts(self, key, pb=pb):
                 yield v

=== modified file 'bzrlib/plugins/launchpad/__init__.py'
--- bzrlib/plugins/launchpad/__init__.py	2011-01-10 22:20:12 +0000
+++ bzrlib/plugins/launchpad/__init__.py	2011-02-11 05:37:04 +0000
@@ -296,11 +296,12 @@
     def run(self, location='.'):
         from bzrlib.plugins.launchpad import lp_api
         from bzrlib.plugins.launchpad.lp_registration import LaunchpadService
-        branch = _mod_branch.Branch.open(location)
+        branch, _ = _mod_branch.Branch.open_containing(location)
         service = LaunchpadService()
         launchpad = lp_api.login(service)
-        lp_branch = lp_api.load_branch(launchpad, branch)
-        lp_branch.requestMirror()
+        lp_branch = lp_api.LaunchpadBranch.from_bzr(launchpad, branch,
+                create_missing=False)
+        lp_branch.lp.requestMirror()
 
 
 register_command(cmd_launchpad_mirror)

=== modified file 'bzrlib/plugins/launchpad/lp_api.py'
--- bzrlib/plugins/launchpad/lp_api.py	2011-02-03 05:13:46 +0000
+++ bzrlib/plugins/launchpad/lp_api.py	2011-02-11 05:27:33 +0000
@@ -35,7 +35,6 @@
     )
 from bzrlib.plugins.launchpad.lp_registration import (
     InvalidLaunchpadInstance,
-    NotLaunchpadBranch,
     )
 
 try:
@@ -197,6 +196,9 @@
         url = bzr_branch.get_push_location()
         if url is not None:
             yield url
+        url = bzr_branch.get_parent()
+        if url is not None:
+            yield url
         yield bzr_branch.base
 
     @staticmethod
@@ -294,29 +296,6 @@
         return self.bzr.repository.revision_tree(lca)
 
 
-def load_branch(launchpad, branch):
-    """Return the launchpadlib Branch object corresponding to 'branch'.
-
-    :param launchpad: The root `Launchpad` object from launchpadlib.
-    :param branch: A `bzrlib.branch.Branch`.
-    :raise NotLaunchpadBranch: If we cannot determine the Launchpad URL of
-        `branch`.
-    :return: A launchpadlib Branch object.
-    """
-    # XXX: This duplicates the "What are possible URLs for the branch that
-    # Launchpad might recognize" logic found in cmd_lp_open.
-
-    # XXX: This makes multiple roundtrips to Launchpad for what is
-    # conceptually a single operation -- get me the branches that match these
-    # URLs. Unfortunately, Launchpad's support for such operations is poor, so
-    # we have to allow multiple roundtrips.
-    for url in branch.get_public_branch(), branch.get_push_location():
-        lp_branch = launchpad.branches.getByUrl(url=url)
-        if lp_branch:
-            return lp_branch
-    raise NotLaunchpadBranch(url)
-
-
 def canonical_url(object):
     """Return the canonical URL for a branch."""
     scheme, netloc, path, params, query, fragment = urlparse.urlparse(

=== modified file 'bzrlib/plugins/weave_fmt/workingtree.py'
--- bzrlib/plugins/weave_fmt/workingtree.py	2011-02-11 17:52:38 +0000
+++ bzrlib/plugins/weave_fmt/workingtree.py	2011-02-18 20:28:34 +0000
@@ -51,6 +51,10 @@
 
     requires_normalized_unicode_filenames = True
 
+    case_sensitive_filename = "Branch-FoRMaT"
+
+    missing_parent_conflicts = False
+
     def get_format_description(self):
         """See WorkingTreeFormat.get_format_description()."""
         return "Working tree format 2"

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2011-02-10 16:30:54 +0000
+++ bzrlib/tests/__init__.py	2011-02-11 17:12:35 +0000
@@ -3858,7 +3858,7 @@
         'bzrlib.tests.test_testament',
         'bzrlib.tests.test_textfile',
         'bzrlib.tests.test_textmerge',
-        'bzrlib.tests.test_thread',
+        'bzrlib.tests.test_cethread',
         'bzrlib.tests.test_timestamp',
         'bzrlib.tests.test_trace',
         'bzrlib.tests.test_transactions',

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2011-01-12 23:33:40 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2011-02-11 05:57:31 +0000
@@ -28,6 +28,7 @@
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests import (
+    fixtures,
     HardlinkFeature,
     test_server,
     )
@@ -270,11 +271,8 @@
 
     def test_branch_fetches_all_tags(self):
         builder = self.make_branch_builder('source')
-        builder.build_commit(message="Rev 1", rev_id='rev-1')
-        builder.build_commit(message="Rev 2", rev_id='rev-2')
-        source = builder.get_branch()
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
         source.tags.set_tag('tag-a', 'rev-2')
-        source.set_last_revision_info(1, 'rev-1')
         # Now source has a tag not in its ancestry.  Make a branch from it.
         self.run_bzr('branch source new-branch')
         new_branch = branch.Branch.open('new-branch')
@@ -466,12 +464,9 @@
     def test_branch_from_branch_with_tags(self):
         self.setup_smart_server_with_call_log()
         builder = self.make_branch_builder('source')
-        builder.build_commit(message="Rev 1", rev_id='rev-1')
-        builder.build_commit(message="Rev 2", rev_id='rev-2')
-        source = builder.get_branch()
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
         source.tags.set_tag('tag-a', 'rev-2')
         source.tags.set_tag('tag-missing', 'missing-rev')
-        source.set_last_revision_info(1, 'rev-1')
         # Now source has a tag not in its ancestry.  Make a branch from it.
         self.reset_smart_call_log()
         out, err = self.run_bzr(['branch', self.get_url('source'), 'target'])

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2010-09-23 02:56:19 +0000
+++ bzrlib/tests/blackbox/test_export.py	2011-02-16 17:20:10 +0000
@@ -148,7 +148,8 @@
         self.assertEqual(['test/a', 'test/b/', 'test/b/c', 'test/d/'], names)
 
         file_attr = stat.S_IFREG | export.zip_exporter.FILE_PERMISSIONS
-        dir_attr = stat.S_IFDIR | export.zip_exporter.ZIP_DIRECTORY_BIT
+        dir_attr = (stat.S_IFDIR | export.zip_exporter.ZIP_DIRECTORY_BIT |
+                    export.zip_exporter.DIR_PERMISSIONS)
 
         a_info = zfile.getinfo(names[0])
         self.assertEqual(file_attr, a_info.external_attr)

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2011-02-07 04:14:29 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2011-02-11 05:57:31 +0000
@@ -29,7 +29,10 @@
 from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
 from bzrlib.osutils import pathjoin
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import (
+    fixtures,
+    TestCaseWithTransport,
+    )
 from bzrlib.uncommit import uncommit
 from bzrlib.workingtree import WorkingTree
 
@@ -148,13 +151,9 @@
         """
         # Make a source, sprout a target off it
         builder = self.make_branch_builder('source')
-        builder.build_commit(message="Rev 1", rev_id='rev-1')
-        source = builder.get_branch()
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
         target_bzrdir = source.bzrdir.sprout('target')
-        # Add a non-ancestry tag to source
-        builder.build_commit(message="Rev 2", rev_id='rev-2')
         source.tags.set_tag('tag-a', 'rev-2')
-        source.set_last_revision_info(1, 'rev-1')
         # Pull from source
         self.run_bzr('pull -d target source')
         target = target_bzrdir.open_branch()

=== modified file 'bzrlib/tests/fixtures.py'
--- bzrlib/tests/fixtures.py	2010-06-26 01:07:16 +0000
+++ bzrlib/tests/fixtures.py	2011-02-09 06:36:35 +0000
@@ -97,3 +97,31 @@
     def __exit__(self, exc_type, exc_val, exc_tb):
         self._calls.append('__exit__')
         return False # propogate exceptions.
+
+
+def build_branch_with_non_ancestral_rev(branch_builder):
+    """Builds a branch with a rev not in the ancestry of the tip.
+
+    This is the revision graph::
+
+      rev-2
+        |
+      rev-1
+        |
+      (null)
+
+    The branch tip is 'rev-1'.  'rev-2' is present in the branch's repository,
+    but is not part of rev-1's ancestry.
+
+    :param branch_builder: A BranchBuilder (e.g. from
+        TestCaseWithMemoryTransport.make_branch_builder).
+    :returns: the new branch
+    """
+    # Make a sequence of two commits
+    branch_builder.build_commit(message="Rev 1", rev_id='rev-1')
+    branch_builder.build_commit(message="Rev 2", rev_id='rev-2')
+    # Move the branch tip back to the first commit
+    source = branch_builder.get_branch()
+    source.set_last_revision_info(1, 'rev-1')
+    return source
+

=== modified file 'bzrlib/tests/per_branch/test_pull.py'
--- bzrlib/tests/per_branch/test_pull.py	2010-12-01 01:07:39 +0000
+++ bzrlib/tests/per_branch/test_pull.py	2011-02-09 06:36:35 +0000
@@ -25,7 +25,11 @@
     memorytree,
     revision,
     )
-from bzrlib.tests import per_branch, TestNotApplicable
+from bzrlib.tests import (
+    fixtures,
+    per_branch,
+    TestNotApplicable,
+    )
 
 
 class TestPull(per_branch.TestCaseWithBranch):
@@ -144,17 +148,14 @@
             builder = self.make_branch_builder('source')
         except errors.UninitializableFormat:
             raise TestNotApplicable('uninitializeable format')
-        builder.build_commit(message="Rev 1", rev_id='rev-1')
-        source = builder.get_branch()
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
         target = source.bzrdir.sprout('target').open_branch()
-        # Add a non-ancestry tag to source
-        builder.build_commit(message="Rev 2", rev_id='rev-2')
+        # Add a tag to the source, then pull from source
         try:
             source.tags.set_tag('tag-a', 'rev-2')
         except errors.TagsNotSupported:
             raise TestNotApplicable('format does not support tags.')
-        source.set_last_revision_info(1, 'rev-1')
-        # Pull from source
+        source.tags.set_tag('tag-a', 'rev-2')
         target.pull(source)
         # The tag is present, and so is its revision.
         self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))
@@ -167,19 +168,15 @@
             builder = self.make_branch_builder('source')
         except errors.UninitializableFormat:
             raise TestNotApplicable('uninitializeable format')
-        builder.build_commit(message="Rev 1", rev_id='rev-1')
-        source = builder.get_branch()
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
         target = source.bzrdir.sprout('target').open_branch()
-        # Add a non-ancestry tag to source
-        builder.build_commit(message="Rev 2", rev_id='rev-2')
-        try:
-            source.tags.set_tag('tag-a', 'rev-2')
-        except errors.TagsNotSupported:
-            raise TestNotApplicable('format does not support tags.')
-        source.set_last_revision_info(1, 'rev-1')
         # Add a new commit to the ancestry
         builder.build_commit(message="Rev 2 again", rev_id='rev-2-again')
-        # Pull from source
+        # Add a tag to the source, then pull rev-2-again from source
+        try:
+            source.tags.set_tag('tag-a', 'rev-2')
+        except errors.TagsNotSupported:
+            raise TestNotApplicable('format does not support tags.')
         target.pull(source, 'rev-2-again')
         # The tag is present, and so is its revision.
         self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2011-01-13 01:52:50 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2011-02-18 20:28:34 +0000
@@ -23,7 +23,7 @@
     errors,
     )
 from bzrlib.revision import NULL_REVISION
-from bzrlib.tests import TestNotApplicable, transport_util
+from bzrlib.tests import fixtures, TestNotApplicable, transport_util
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
 
@@ -176,17 +176,20 @@
 
     def test_unstack_fetches(self):
         """Removing the stacked-on branch pulls across all data"""
+        try:
+            builder = self.make_branch_builder('trunk')
+        except errors.UninitializableFormat:
+            raise TestNotApplicable('uninitializeable format')
         # We have a mainline
-        trunk_tree = self.make_branch_and_tree('mainline')
-        trunk_revid = trunk_tree.commit('revision on mainline')
-        # and make branch from it which is stacked
+        trunk = fixtures.build_branch_with_non_ancestral_rev(builder)
+        mainline_revid = 'rev-1'
+        # and make branch from it which is stacked (with no tags)
         try:
-            new_dir = trunk_tree.bzrdir.sprout(self.get_url('newbranch'),
-                stacked=True)
+            new_dir = trunk.bzrdir.sprout(self.get_url('newbranch'), stacked=True)
         except unstackable_format_errors, e:
             raise TestNotApplicable(e)
         # stacked repository
-        self.assertRevisionNotInRepository('newbranch', trunk_revid)
+        self.assertRevisionNotInRepository('newbranch', mainline_revid)
         # TODO: we'd like to commit in the stacked repository; that requires
         # some care (maybe a BranchBuilder) if it's remote and has no
         # workingtree
@@ -195,13 +198,21 @@
         # now when we unstack that should implicitly fetch, to make sure that
         # the branch will still work
         new_branch = new_dir.open_branch()
+        try:
+            new_branch.tags.set_tag('tag-a', 'rev-2')
+        except errors.TagsNotSupported:
+            tags_supported = False
+        else:
+            tags_supported = True
         new_branch.set_stacked_on_url(None)
-        self.assertRevisionInRepository('newbranch', trunk_revid)
+        self.assertRevisionInRepository('newbranch', mainline_revid)
         # of course it's still in the mainline
-        self.assertRevisionInRepository('mainline', trunk_revid)
+        self.assertRevisionInRepository('trunk', mainline_revid)
+        if tags_supported:
+            # the tagged revision in trunk is now in newbranch too
+            self.assertRevisionInRepository('newbranch', 'rev-2')
         # and now we're no longer stacked
-        self.assertRaises(errors.NotStacked,
-            new_branch.get_stacked_on_url)
+        self.assertRaises(errors.NotStacked, new_branch.get_stacked_on_url)
 
     def test_unstack_already_locked(self):
         """Removing the stacked-on branch with an already write-locked branch

=== modified file 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_controldir/test_controldir.py	2011-02-08 15:35:14 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2011-02-18 20:28:34 +0000
@@ -33,6 +33,7 @@
     )
 import bzrlib.revision
 from bzrlib.tests import (
+    fixtures,
     ChrootedTestCase,
     TestNotApplicable,
     TestSkipped,
@@ -675,14 +676,11 @@
         # when sprouting a branch all revisions named in the tags are copied
         # too.
         builder = self.make_branch_builder('source')
-        builder.build_commit(message="Rev 1", rev_id='rev-1')
-        builder.build_commit(message="Rev 2", rev_id='rev-2')
-        source = builder.get_branch()
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
         try:
             source.tags.set_tag('tag-a', 'rev-2')
         except errors.TagsNotSupported:
             raise TestNotApplicable('Branch format does not support tags.')
-        source.set_last_revision_info(1, 'rev-1')
         # Now source has a tag not in its ancestry.  Sprout its controldir.
         dir = source.bzrdir
         target = dir.sprout(self.get_url('target'))

=== modified file 'bzrlib/tests/per_repository_reference/test_graph.py'
--- bzrlib/tests/per_repository_reference/test_graph.py	2011-02-08 08:01:40 +0000
+++ bzrlib/tests/per_repository_reference/test_graph.py	2011-02-09 08:07:51 +0000
@@ -29,7 +29,7 @@
         See <https://bugs.launchpad.net/bugs/715000>.
         """
         branch_a, branch_b, branch_c, revid_1 = self.make_double_stacked_branches()
-        for br in [branch_c]:
+        for br in [branch_a, branch_b, branch_c]:
             self.assertEquals(
                 [revid_1],
                 br.repository.get_known_graph_ancestry([revid_1]).topo_sort())

=== modified file 'bzrlib/tests/per_workingtree/test_pull.py'
--- bzrlib/tests/per_workingtree/test_pull.py	2011-01-11 16:36:43 +0000
+++ bzrlib/tests/per_workingtree/test_pull.py	2011-02-18 20:28:34 +0000
@@ -15,7 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 
 from bzrlib import tests
 from bzrlib.tests import per_workingtree
@@ -85,10 +84,10 @@
         return builder.get_branch()
 
     def test_pull_orphans(self):
-        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
-        if isinstance(self.workingtree_format, WorkingTreeFormat2):
+        if not self.workingtree_format.missing_parent_conflicts:
             raise tests.TestSkipped(
-                'WorkingTreeFormat2 does not support missing parent conflicts')
+                '%r does not support missing parent conflicts' %
+                    self.workingtree_format)
         trunk = self.make_branch_deleting_dir('trunk')
         work = trunk.bzrdir.sprout('work', revision_id='2').open_workingtree()
         work.branch.get_config().set_user_option(

=== renamed file 'bzrlib/tests/test_thread.py' => 'bzrlib/tests/test_cethread.py'
--- bzrlib/tests/test_thread.py	2011-02-08 16:26:23 +0000
+++ bzrlib/tests/test_cethread.py	2011-02-11 16:15:39 +0000
@@ -17,8 +17,8 @@
 import threading
 
 from bzrlib import (
+    cethread,
     tests,
-    thread,
     )
 
 
@@ -28,7 +28,7 @@
         def do_nothing():
             pass
 
-        tt = thread.CatchingExceptionThread(target=do_nothing)
+        tt = cethread.CatchingExceptionThread(target=do_nothing)
         tt.start()
         tt.join()
 
@@ -39,11 +39,11 @@
         def raise_my_exception():
             raise MyException()
 
-        tt = thread.CatchingExceptionThread(target=raise_my_exception)
+        tt = cethread.CatchingExceptionThread(target=raise_my_exception)
         tt.start()
         self.assertRaises(MyException, tt.join)
 
-    def test_join_when_no_exception(self):
+    def test_join_around_exception(self):
         resume = threading.Event()
         class MyException(Exception):
             pass
@@ -54,11 +54,108 @@
             # Now we can raise
             raise MyException()
 
-        tt = thread.CatchingExceptionThread(target=raise_my_exception)
+        tt = cethread.CatchingExceptionThread(target=raise_my_exception)
         tt.start()
         tt.join(timeout=0)
         self.assertIs(None, tt.exception)
         resume.set()
         self.assertRaises(MyException, tt.join)
 
-
+    def test_sync_event(self):
+        control = threading.Event()
+        in_thread = threading.Event()
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            # Wait for the test to tell us to resume
+            control.wait()
+            # Now we can raise
+            raise MyException()
+
+        tt = cethread.CatchingExceptionThread(target=raise_my_exception,
+                                            sync_event=in_thread)
+        tt.start()
+        tt.join(timeout=0)
+        self.assertIs(None, tt.exception)
+        self.assertIs(in_thread, tt.sync_event)
+        control.set()
+        self.assertRaises(MyException, tt.join)
+        self.assertEquals(True, tt.sync_event.isSet())
+
+    def test_switch_and_set(self):
+        """Caller can precisely control a thread."""
+        control1 = threading.Event()
+        control2 = threading.Event()
+        control3 = threading.Event()
+
+        class TestThread(cethread.CatchingExceptionThread):
+
+            def __init__(self):
+                super(TestThread, self).__init__(target=self.step_by_step)
+                self.current_step = 'starting'
+                self.step1 = threading.Event()
+                self.set_sync_event(self.step1)
+                self.step2 = threading.Event()
+                self.final = threading.Event()
+
+            def step_by_step(self):
+                control1.wait()
+                self.current_step = 'step1'
+                self.switch_and_set(self.step2)
+                control2.wait()
+                self.current_step = 'step2'
+                self.switch_and_set(self.final)
+                control3.wait()
+                self.current_step = 'done'
+
+        tt = TestThread()
+        tt.start()
+        self.assertEquals('starting', tt.current_step)
+        control1.set()
+        tt.step1.wait()
+        self.assertEquals('step1', tt.current_step)
+        control2.set()
+        tt.step2.wait()
+        self.assertEquals('step2', tt.current_step)
+        control3.set()
+        # We don't wait on tt.final
+        tt.join()
+        self.assertEquals('done', tt.current_step)
+
+    def test_exception_while_switch_and_set(self):
+        control1 = threading.Event()
+
+        class MyException(Exception):
+            pass
+
+        class TestThread(cethread.CatchingExceptionThread):
+
+            def __init__(self, *args, **kwargs):
+                self.step1 = threading.Event()
+                self.step2 = threading.Event()
+                super(TestThread, self).__init__(target=self.step_by_step,
+                                                 sync_event=self.step1)
+                self.current_step = 'starting'
+                self.set_sync_event(self.step1)
+
+            def step_by_step(self):
+                control1.wait()
+                self.current_step = 'step1'
+                self.switch_and_set(self.step2)
+
+            def set_sync_event(self, event):
+                # We force an exception while trying to set step2
+                if event is self.step2:
+                    raise MyException()
+                super(TestThread, self).set_sync_event(event)
+
+        tt = TestThread()
+        tt.start()
+        self.assertEquals('starting', tt.current_step)
+        control1.set()
+        # We now wait on step1 which will be set when catching the exception
+        tt.step1.wait()
+        self.assertRaises(MyException, tt.pending_exception)
+        self.assertIs(tt.step1, tt.sync_event)
+        self.assertTrue(tt.step1.isSet())

=== modified file 'bzrlib/tests/test_decorators.py'
--- bzrlib/tests/test_decorators.py	2010-04-23 08:51:52 +0000
+++ bzrlib/tests/test_decorators.py	2011-02-14 11:57:45 +0000
@@ -27,11 +27,13 @@
     pass
 
 
-def create_decorator_sample(style, unlock_error=None):
+def create_decorator_sample(style, unlock_error=None, meth=None):
     """Create a DecoratorSample object, using specific lock operators.
 
     :param style: The type of lock decorators to use (fast/pretty/None)
     :param unlock_error: If specified, an error to raise from unlock.
+    :param meth: a function to be decorated and added as a 'meth_read' and
+        'meth_write' to the object.
     :return: An instantiated DecoratorSample object.
     """
 
@@ -92,6 +94,10 @@
             self.actions.append('fail_during_write')
             raise TypeError('during write')
 
+        if meth is not None:
+            meth_read = needs_read_lock(meth)
+            meth_write = needs_write_lock(meth)
+
     return DecoratorSample()
 
 
@@ -149,6 +155,20 @@
         self.assertEqual(['lock_write', ('bank', 'bar', 'bing'),
                           'unlock_fail'], sam.actions)
 
+    def test_read_lock_preserves_default_str_kwarg_identity(self):
+        a_constant = 'A str used as a constant'
+        def meth(self, param=a_constant):
+            return param
+        sam = create_decorator_sample(self._decorator_style, meth=meth)
+        self.assertIs(a_constant, sam.meth_read())
+
+    def test_write_lock_preserves_default_str_kwarg_identity(self):
+        a_constant = 'A str used as a constant'
+        def meth(self, param=a_constant):
+            return param
+        sam = create_decorator_sample(self._decorator_style, meth=meth)
+        self.assertIs(a_constant, sam.meth_write())
+
 
 class TestFastDecoratorActions(TestDecoratorActions):
 

=== modified file 'bzrlib/tests/test_hooks.py'
--- bzrlib/tests/test_hooks.py	2010-09-21 03:20:09 +0000
+++ bzrlib/tests/test_hooks.py	2011-01-23 00:24:17 +0000
@@ -19,14 +19,17 @@
 from bzrlib import (
     branch,
     errors,
+    pyutils,
     tests,
     )
 from bzrlib.hooks import (
     HookPoint,
     Hooks,
+    install_lazy_named_hook,
     known_hooks,
     known_hooks_key_to_object,
     known_hooks_key_to_parent_and_attribute,
+    _lazy_hooks,
     )
 from bzrlib.symbol_versioning import (
     deprecated_in,
@@ -112,6 +115,52 @@
         hooks.install_named_hook('set_rh', None, "demo")
         self.assertEqual("demo", hooks.get_hook_name(None))
 
+    hooks = Hooks("bzrlib.tests.test_hooks", "TestHooks.hooks")
+
+    def test_install_lazy_named_hook(self):
+        # When the hook points are not yet registered the hook is
+        # added to the _lazy_hooks dictionary in bzrlib.hooks.
+        self.hooks.add_hook('set_rh', "doc", (0, 15))
+        set_rh = lambda: None
+        install_lazy_named_hook('bzrlib.tests.test_hooks',
+            'TestHooks.hooks', 'set_rh', set_rh, "demo")
+        set_rh_lazy_hooks = _lazy_hooks[
+            ('bzrlib.tests.test_hooks', 'TestHooks.hooks', 'set_rh')]
+        self.assertEquals(1, len(set_rh_lazy_hooks))
+        self.assertEquals(set_rh, set_rh_lazy_hooks[0][0].get_obj())
+        self.assertEquals("demo", set_rh_lazy_hooks[0][1])
+        self.assertEqual(list(TestHooks.hooks['set_rh']), [set_rh])
+
+    set_rh = lambda: None
+
+    def test_install_named_hook_lazy(self):
+        hooks = Hooks()
+        hooks['set_rh'] = HookPoint("set_rh", "doc", (0, 15), None)
+        hooks.install_named_hook_lazy('set_rh', 'bzrlib.tests.test_hooks',
+            'TestHooks.set_rh', "demo")
+        self.assertEqual(list(hooks['set_rh']), [TestHooks.set_rh])
+
+    def test_install_named_hook_lazy_old(self):
+        # An exception is raised if a lazy hook is raised for
+        # an old style hook point.
+        hooks = Hooks()
+        hooks['set_rh'] = []
+        self.assertRaises(errors.UnsupportedOperation,
+            hooks.install_named_hook_lazy,
+            'set_rh', 'bzrlib.tests.test_hooks', 'TestHooks.set_rh',
+            "demo")
+
+    def test_valid_lazy_hooks(self):
+        # Make sure that all the registered lazy hooks are referring to existing
+        # hook points which allow lazy registration.
+        for key, callbacks in _lazy_hooks.iteritems():
+            (module_name, member_name, hook_name) = key
+            obj = pyutils.get_named_object(module_name, member_name)
+            self.assertEquals(obj._module, module_name)
+            self.assertEquals(obj._member_name, member_name)
+            self.assertTrue(hook_name in obj)
+            self.assertIs(callbacks, obj[hook_name]._callbacks)
+
 
 class TestHook(tests.TestCase):
 
@@ -144,6 +193,16 @@
         hook.hook(callback, "my callback")
         self.assertEqual([callback], list(hook))
 
+    def lazy_callback():
+        pass
+
+    def test_lazy_hook(self):
+        hook = HookPoint("foo", "no docs", None, None)
+        hook.hook_lazy(
+            "bzrlib.tests.test_hooks", "TestHook.lazy_callback",
+            "my callback")
+        self.assertEqual([TestHook.lazy_callback], list(hook))
+
     def test___repr(self):
         # The repr should list all the callbacks, with names.
         hook = HookPoint("foo", "no docs", None, None)

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2011-02-08 16:26:23 +0000
+++ bzrlib/tests/test_http.py	2011-02-10 12:37:27 +0000
@@ -32,12 +32,12 @@
 import bzrlib
 from bzrlib import (
     bzrdir,
+    cethread,
     config,
     errors,
     osutils,
     remote as _mod_remote,
     tests,
-    thread,
     transport,
     ui,
     )

=== modified file 'bzrlib/tests/test_server.py'
--- bzrlib/tests/test_server.py	2011-02-08 16:26:23 +0000
+++ bzrlib/tests/test_server.py	2011-02-10 12:37:27 +0000
@@ -22,8 +22,8 @@
 
 
 from bzrlib import (
+    cethread,
     osutils,
-    thread,
     transport,
     urlutils,
     )
@@ -243,7 +243,7 @@
         raise NotImplementedError
 
 
-class TestThread(thread.CatchingExceptionThread):
+class TestThread(cethread.CatchingExceptionThread):
 
     def join(self, timeout=5):
         """Overrides to use a default timeout.

=== modified file 'bzrlib/version_info_formats/format_rio.py'
--- bzrlib/version_info_formats/format_rio.py	2010-02-17 17:11:16 +0000
+++ bzrlib/version_info_formats/format_rio.py	2011-02-18 19:01:43 +0000
@@ -84,10 +84,12 @@
     """Hooks for rio-formatted version-info output."""
 
     def __init__(self):
-        super(RioVersionInfoBuilderHooks, self).__init__()
-        self.create_hook(hooks.HookPoint('revision',
+        super(RioVersionInfoBuilderHooks, self).__init__(
+            "bzrlib.version_info_formats.format_rio", "RioVersionInfoBuilder.hooks")
+        self.add_hook('revision',
             "Invoked when adding information about a revision to the"
             " RIO stanza that is printed. revision is called with a"
-            " revision object and a RIO stanza.", (1, 15), None))
+            " revision object and a RIO stanza.", (1, 15), None)
+
 
 RioVersionInfoBuilder.hooks = RioVersionInfoBuilderHooks()

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2011-02-09 06:35:00 +0000
+++ bzrlib/versionedfile.py	2011-02-14 02:37:04 +0000
@@ -1,7 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
-#
-# Authors:
-#   Johan Rydberg <jrydberg@gnu.org>
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -930,6 +927,10 @@
 
     The use of tuples allows a single code base to support several different
     uses with only the mapping logic changing from instance to instance.
+
+    :ivar _immediate_fallback_vfs: For subclasses that support stacking,
+        this is a list of other VersionedFiles immediately underneath this
+        one.  They may in turn each have further fallbacks.
     """
 
     def add_lines(self, key, parents, lines, parent_texts=None,
@@ -1201,7 +1202,7 @@
         at open time because they may change after the objects are opened.
         """
         all_fallbacks = []
-        for a_vfs in self._fallback_vfs:
+        for a_vfs in self._immediate_fallback_vfs:
             all_fallbacks.append(a_vfs)
             all_fallbacks.extend(a_vfs._transitive_fallbacks())
         return all_fallbacks

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-02-11 17:52:38 +0000
+++ bzrlib/workingtree.py	2011-02-18 20:28:34 +0000
@@ -2795,6 +2795,9 @@
 
     case_sensitive_filename = "FoRMaT"
 
+    missing_parent_conflicts = False
+    """If this format supports missing parent conflicts."""
+
     @classmethod
     def find_format(klass, a_bzrdir):
         """Return the format for the working tree object in a_bzrdir."""
@@ -2883,6 +2886,8 @@
 
     upgrade_recommended = True
 
+    missing_parent_conflicts = True
+
     def get_format_string(self):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar-NG Working Tree format 3"

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2011-01-25 23:21:46 +0000
+++ bzrlib/workingtree_4.py	2011-02-11 17:58:56 +0000
@@ -1406,6 +1406,8 @@
 
 class DirStateWorkingTreeFormat(WorkingTreeFormat3):
 
+    missing_parent_conflicts = True
+
     def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
                    accelerator_tree=None, hardlink=False):
         """See WorkingTreeFormat.initialize().

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-02-10 16:30:54 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-02-18 17:05:11 +0000
@@ -50,6 +50,9 @@
 * ``bzr annotate -r-1 file`` will now properly annotate a deleted file.
   (Andrew King, #537442)
 
+* ``bzr export`` to zip files will now set a mode on directories.
+  (Jelmer Vernooij, #207253)
+
 * ``bzr push --overwrite`` with an older revision specified will now correctly
   roll back the target branch. (Jelmer Vernooij, #386576)
 
@@ -68,6 +71,10 @@
   paths, however they may still print junk if not on a UTF-8 terminal.
   (Martin [gz], #707954)
 
+* ``bzr reconfigure --unstacked`` now copies revisions (and their
+  ancestors) named in tags into the unstacked repository, not just the
+  ancestry of the branch's tip.  (Andrew Bennetts, #401646)
+
 * ``bzr serve`` no longer crashes when a server_started hook is installed and
   IPv6 support is available on the system. (Jelmer Vernooij, #293697)
 
@@ -86,9 +93,16 @@
   ``bzr plugins`` and in crash reports.
   (#704195, Martin Pool)
 
+* The "pretty" version of ``needs_read_lock`` and ``needs_write_lock`` now
+  preserves the identity of default parameter values.
+  (Andrew Bennetts, #718569)
+
 * ``bzr dump-btree --raw`` no longer tracebacks on a B-Tree file
   containing no rows. (Eric Siegerman, #715508)
 
+* Fix ``bzr lp-mirror`` to work on command line branch URLs and branches
+  without an explicit public location. (Max Bowsher)
+
 Documentation
 *************
 
@@ -128,8 +142,11 @@
 * ``CatchingExceptionThread`` (formerly ThreadWithException) has been moved
   out of the ``bzrlib.tests`` hierarchy to make it clearer that it can be used
   outside of tests. This class makes it easier to track exceptions in threads
-  by cacthing them so they can be re-raised in the controlling thread.
-  (Vincent Ladeuil)
+  by cacthing them so they can be re-raised in the controlling thread. It's
+  available in the ``bzrlib.cethread`` module.  (Vincent Ladeuil)
+
+* ``HookPoint.lazy_hook`` and ``Hooks.install_named_lazy_hook`` can install 
+  hooks for which the callable is loaded lazily.  (Jelmer Vernooij)
 
 Testing
 *******

=== modified file 'doc/en/user-guide/hooks.txt'
--- doc/en/user-guide/hooks.txt	2010-10-08 10:50:51 +0000
+++ doc/en/user-guide/hooks.txt	2011-02-18 18:19:50 +0000
@@ -54,6 +54,20 @@
 itself.  The third argument is a name ``'My post_push hook'``, which can be
 used in progress messages and error messages.
 
+To reduce the start-up time of Bazaar it is also possible to "lazily" install hooks,
+using the ``bzrlib.hooks.install_lazy_named_hook`` function. This removes the need
+to load the module that contains the hook point just to install the hook. Here's lazy
+version of the example above::
+
+    from bzrlib import hooks
+
+    def post_push_hook(push_result):
+        print "The new revno is %d" % push_result.new_revno
+
+
+    hooks.install_lazy_named_hook('bzrlib.branch', 'Branch.hooks',
+        'post_push', post_push_hook, 'My post_push hook')
+
 Debugging hooks
 ---------------
 

